package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnum;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnumManager;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgSettingsRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgSettingsEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamOrgSettingsInfo;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月17日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class ExamServiceImpl implements ExamService {
	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamStudentServiceImpl examStudentService;

	@Autowired
	ExamOrgSettingsRepo examOrgSettingsRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamPropertyRepo examPropertyRepo;

	@Autowired
	ExamOrgPropertyRepo examOrgPropertyRepo;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examName
	 * @param rootOrgId
	 * @return
	 */
	public ExamEntity findExamByNameAndRootOrgId(String examName, Long rootOrgId) {
		ExamEntity exam = new ExamEntity();
		exam.setName(examName);
		exam.setRootOrgId(rootOrgId);
		return examRepo.findOne(Example.of(exam));
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExam(cn.com.
	 * qmth.examcloud.core.examwork.service.bean.ExamInfo)
	 */
	@Override
	public ExamEntity saveExam(ExamInfo examInfo) {

		ExamEntity exam = new ExamEntity();
		exam.setId(examInfo.getId());
		exam.setBeginTime(examInfo.getBeginTime());
		exam.setDuration(examInfo.getDuration());
		exam.setEnable(examInfo.getEnable());
		exam.setEndTime(examInfo.getEndTime());
		exam.setExamTimes(examInfo.getExamTimes());
		exam.setExamType(examInfo.getExamType());
		exam.setId(examInfo.getId());
		exam.setName(examInfo.getName());
		exam.setRemark(examInfo.getRemark());
		exam.setRootOrgId(examInfo.getRootOrgId());

		if (null == exam.getExamType()) {
			throw new StatusException("E-001005", "考试类型为空");
		}

		Map<String, String> properties = examInfo.getProperties();
		Map<DynamicEnum, String> map = checkAndGetExamProperties(properties);

		ExamEntity findOne = examRepo.findByNameAndRootOrgId(exam.getName(), exam.getRootOrgId());
		if (null != findOne && !findOne.getId().equals(exam.getId())) {
			throw new StatusException("E-001001", "考试名称已存在");
		}

		ExamEntity saved = examRepo.save(exam);

		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			ExamPropertyEntity entity = examPropertyRepo.findByexamIdAndKeyId(saved.getId(),
					de.getId());
			if (null == entity) {
				entity = new ExamPropertyEntity();
				entity.setExamId(saved.getId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			examPropertyRepo.save(entity);
		}

		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param properties
	 * @return
	 */
	private Map<DynamicEnum, String> checkAndGetExamProperties(Map<String, String> properties) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();

		Map<DynamicEnum, String> map = Maps.newHashMap();
		if (null == properties) {
			return map;
		}

		for (Entry<String, String> entry : properties.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			if (StringUtils.isBlank(value)) {
				continue;
			}
			DynamicEnum de = null;
			try {
				de = manager.getByName(key);
			} catch (Exception e) {
				throw new StatusException("E-001004", "考试属性错误");
			}
			map.put(de, value.trim());
		}

		String beforeExamRemark = properties.get("BEFORE_EXAM_REMARK");
		if (null != beforeExamRemark && beforeExamRemark.length() > 250000) {
			throw new StatusException("E-001002", "考前说明内容过大");
		}

		String afterExamRemark = properties.get("AFTER_EXAM_REMARK");
		if (null != afterExamRemark && afterExamRemark.length() > 250000) {
			throw new StatusException("E-001002", "考后说明内容过大");
		}

		String cheatingRemark = properties.get("CHEATING_REMARK");
		if (null != cheatingRemark && cheatingRemark.length() > 250000) {
			throw new StatusException("E-001002", "作弊说明内容过大");
		}

		return map;
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExamOrg(cn.
	 * com.qmth.examcloud.core.examwork.service.bean.ExamOrgInfo)
	 */
	@Override
	public ExamOrgSettingsEntity saveExamOrg(ExamOrgSettingsInfo examOrgInfo) {
		ExamOrgSettingsEntity examOrgEntity = null;

		if (null == examOrgInfo.getId()) {
			examOrgEntity = examOrgSettingsRepo.findByExamIdAndOrgId(examOrgInfo.getExamId(),
					examOrgInfo.getOrgId());
			if (null == examOrgEntity) {
				examOrgEntity = new ExamOrgSettingsEntity();
			}
		} else {
			examOrgEntity = examOrgSettingsRepo.findOne(examOrgInfo.getId());
			if (null == examOrgEntity) {
				throw new StatusException("E-001101", "id is wrong");
			}
		}

		examOrgEntity.setBeginTime(examOrgInfo.getBeginTime());
		examOrgEntity.setEndTime(examOrgInfo.getEndTime());
		examOrgEntity.setRootOrgId(examOrgInfo.getRootOrgId());
		examOrgEntity.setExamId(examOrgInfo.getExamId());
		examOrgEntity.setOrgId(examOrgInfo.getOrgId());

		Map<DynamicEnum, String> map = checkAndGetExamProperties(examOrgInfo.getProperties());

		ExamOrgSettingsEntity saved = examOrgSettingsRepo.save(examOrgEntity);

		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			ExamOrgPropertyEntity entity = examOrgPropertyRepo
					.findByexamIdAndOrgIdAndKeyId(saved.getExamId(), saved.getOrgId(), de.getId());
			if (null == entity) {
				entity = new ExamOrgPropertyEntity();
				entity.setExamId(saved.getExamId());
				entity.setOrgId(saved.getOrgId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			examOrgPropertyRepo.save(entity);
		}

		return saved;
	}

	@Override
	public String getOrgProperty(Long examId, Long orgId, String key) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName(key);

		if (null != orgId) {
			ExamOrgPropertyEntity examOrgPropertyEntity = examOrgPropertyRepo
					.findByexamIdAndOrgIdAndKeyId(examId, orgId, de.getId());
			if (null != examOrgPropertyEntity) {
				return examOrgPropertyEntity.getValue();
			}
		}

		ExamPropertyEntity examPropertyEntity = examPropertyRepo.findByexamIdAndKeyId(examId,
				de.getId());
		if (null == examPropertyEntity) {
			return null;
		}

		return examPropertyEntity.getValue();
	}
}
